package keys

import (
	"encoding/json"
	"errors"
	"etcd/etcdsdk/model"
	"etcd/program/common"
	"etcd/program/models"
	"etcd/util/response"
	"fmt"
	"github.com/gin-gonic/gin"
	"log"
	"os/exec"
)

// KeysController key控制器
type KeysController struct {
}

// ReqKeyBody 添加和修改key请求body
type ReqKeyBody struct {
	Path  string `json:"path"`
	Value string `json:"value"`
}

// List 获取目录下key列表
func (api *KeysController) List(c *gin.Context) {
	utilGin := response.Gin{Ctx: c}
	path := c.Query("path")
	log.Println(path)
	var err error
	defer func() {
		if err != nil {
			utilGin.Response(400, err.Error(), nil)
		}
	}()
	client, err := common.GetEtcdClientByGinContext(c)
	if err != nil {
		return
	}
	oldlist, err := client.List(path)
	if err != nil {
		return
	}

	var list []*model.Node
	for _, info := range oldlist {
		newList, err := client.List(info.Path)
		if err != nil {
			return
		}
		list = append(list, newList...)
	}

	m := models.ServiceLst{}
	serviceList, err := m.All()
	if err != nil {
		return
	}


	for _, node := range list {
		isIn := false
		for _, service := range serviceList {
			if node.Path == service.ServicePath {
				isIn = true
				continue
			}
		}
		state := 0
		switch node.Value {
		case "RUNNING":
			state = 1
		case "PAUSE":
			state = 2
		case "STOP":
			state = 0
		}
		////数据库不存在该节点数据 新增
		//if !isIn {
		//	pathList := strings.Split(node.Path, "/")
		//	mNew := &models.ServiceLst{
		//		ServicePath: node.Path,
		//		ServiceName: node.Name,
		//		State:       state,
		//		ServiceIp:   pathList[2],
		//	}
		//	mNew.Insert()
		//}
		//同步节点状态
		if isIn {
			mNew, err := m.FindById(node.Path)
			if err != nil {
				fmt.Println(err)
				return
			}
			mNew.State = state

			mNew.Update()
		}
	}
	for _, service := range serviceList {
		isIn := false
		fmt.Println(service)
		for _, node := range list {
			fmt.Println(node)
			if node.Path == service.ServicePath {
				isIn = true
				continue
			}
		}
		//不在节点内 数据库状态重置为0
		if !isIn {
			service.State = 0
			service.Update()
		}
	}

	serviceList, err = m.All()
	if err != nil {
		return
	}


	utilGin.ResponseTotal(200, "成功", serviceList, int64(len(serviceList)))
}

// Val 获取一个key的值
func (api *KeysController) Val(c *gin.Context) {
	utilGin := response.Gin{Ctx: c}
	path := c.Query("path")
	log.Println(path)
	var err error
	defer func() {
		if err != nil {
			utilGin.Response(400, err.Error(), nil)
		}
	}()
	client, err := common.GetEtcdClientByGinContext(c)
	if err != nil {
		return
	}
	list, err := client.Val(path)
	if err != nil {
		return
	}

	utilGin.Response(200, "成功", list)
}

// Add 添加key
func (api *KeysController) Add(c *gin.Context) {
	utilGin := response.Gin{Ctx: c}
	req := &models.ServiceLst{}
	data, _ := c.GetRawData()
	json.Unmarshal(data, req)

	var err error
	defer func() {
		if err != nil {
			utilGin.Response(400, err.Error(), nil)
		}
	}()
	client, err := common.GetEtcdClientByGinContext(c)
	if err != nil {
		return
	}

	//oldlist, err := client.List("/services")
	//if err != nil {
	//	return
	//}

	//var list []*model.Node
	//for _, info := range oldlist {
	//	newList, err := client.List(info.Path)
	//	if err != nil {
	//		return
	//	}
	//	list = append(list, newList...)
	//}

	//for _, info := range list{
	//	if info.Path == req.ServicePath{
	//		err = errors.New("该节点已经在etcd列表中，请手动维护etcd列表")
	//		return
	//	}
	//}

	m := models.ServiceLst{}
	serviceList, err := m.All()
	if err != nil {
		return
	}

	for _, info := range serviceList{
		if info.ExePath == req.ExePath{
			err = errors.New("该服务路径已经存在，请修改")
			return
		}
	}

	node, _ := client.Val(req.ServicePath)
	if node != nil {
		switch node.Value {
		case "RUNNING":
			req.State = 1
		case "PAUSE":
			req.State = 2
		case "STOP":
			req.State = 0
		}
		err = req.Update()
		if err != nil {
			return
		}
	}

	err = req.Insert()
	if err != nil {
		return
	}

	utilGin.Response(200, "成功", req)
}

// Put 修改key
func (api *KeysController) Put(c *gin.Context) {
	utilGin := response.Gin{Ctx: c}
	req := &models.ServiceLst{}
	data, _ := c.GetRawData()
	json.Unmarshal(data, req)

	var err error
	defer func() {
		if err != nil {
			utilGin.Response(400, err.Error(), nil)
		}
	}()
	err = req.Update()
	if err != nil {
		return
	}

	utilGin.Response(200, "成功", req)
}

// Del 删除key
func (api *KeysController) Del(c *gin.Context) {
	utilGin := response.Gin{Ctx: c}
	path := c.Query("service_path")
	var err error
	defer func() {
		if err != nil {
			utilGin.Response(400, err.Error(), nil)
		}
	}()
	if path == "" {
		err = errors.New("Path cannot be empty")
		return
	}
	//client, err := common.GetEtcdClientByGinContext(c)
	//if err != nil {
	//	return
	//}
	//err = client.Del(path)
	//if err != nil {
	//	return
	//}
	m := models.ServiceLst{}
	err = m.Del(path)
	if err != nil {
		return
	}

	utilGin.Response(200, "成功", nil)
}

// Members 获取etcd服务节点
func (api *KeysController) Members(c *gin.Context) {
	utilGin := response.Gin{Ctx: c}
	var err error
	defer func() {
		if err != nil {
			utilGin.Response(400, err.Error(), nil)
		}
	}()
	client, err := common.GetEtcdClientByGinContext(c)
	if err != nil {
		return
	}
	members, err := client.Members()
	if err != nil {
		return
	}

	utilGin.Response(200, "成功", members)
}

func (api *KeysController) Cmd(c *gin.Context) {
	utilGin := response.Gin{Ctx: c}
	path := c.Query("service_path")
	cmd := c.Query("cmd")
	var err error
	defer func() {
		if err != nil {
			//c.JSON(http.StatusBadRequest, gin.H{
			//	"msg": err.Error(),
			//})
			utilGin.Response(400, err.Error(), nil)
		}
	}()
	if path == "" || cmd == "" {
		err = errors.New("Path cannot be empty")
		return
	}

	m := models.ServiceLst{}
	now, err := m.FindById(path)
	if err != nil {
		return
	}

	if now.ExePath == ""{
		err = errors.New("服务路径不能为空")
		return
	}

	client, err := common.GetEtcdClientByGinContext(c)
	if err != nil {
		return
	}
	node, err := client.Val(path)
	fmt.Println(node)
	if err == nil && node.IsDir {
		err = errors.New("服务路径必须是包含可执行文件的完整路径")
		return
	}

	state := 0
	switch cmd {
	case "start":
		//if now.State != 0{
		//	utilGin := response.Gin{Ctx: c}
		//	utilGin.Response(400, "请先停止服务", now)
		//	return
		//}
		state = 1
		err = checkExe("install", now.ExePath)
		fmt.Println(err)
		err = checkExe(cmd, now.ExePath)
		if err != nil {
			return
		}
		err = client.Put(path, []byte("RUNNING"))
		if err != nil {
			return
		}
	case "pause":
		//if now.State != 1{
		//	utilGin := response.Gin{Ctx: c}
		//	utilGin.Response(400, "请先启动服务", now)
		//	return
		//}
		state = 2
		err = client.Put(path, []byte("PAUSE"))
		if err != nil {
			return
		}
	case "recover":
		//if now.State != 2{
		//	utilGin := response.Gin{Ctx: c}
		//	utilGin.Response(400, "请先暂停服务", now)
		//	return
		//}
		state = 1
		err = client.Put(path, []byte("RUNNING"))
		if err != nil {
			return
		}
	case "stop":
		//if now.State != 1 || now.State != 2{
		//	utilGin := response.Gin{Ctx: c}
		//	utilGin.Response(400, "请先启动服务", now)
		//	return
		//}
		state = 0
		err = client.Put(path, []byte("STOP"))
		if err != nil {
			return
		}
		err = checkExe(cmd, now.ExePath)
		if err != nil {
			return
		}

	}

	now.State = state
	now.Update()


	utilGin.Response(200, "成功", now)
}

func checkExe(cmd, exeAdress string) (err error) {
	c := exec.Command("cmd.exe", "/c", exeAdress+" "+cmd)
	if err = c.Run(); err != nil {
		fmt.Println("Error: ", err)
		err = errors.New("请检查您的服务路径配置")
	}
	return err
}
